use encoding_rs::{DecoderResult, UTF_8};

pub fn validate<I: Iterator<Item = u8>>(iter: I) -> bool {
    let mut decoder = UTF_8.new_decoder();

    let mut dest = [0; 4];
    for b in iter {
        let (status, _, _) = decoder.decode_to_utf8_without_replacement(&[b], &mut dest, false);

        if !valid_result(status) {
            return false;
        }
    }

    let (status, _, _) = decoder.decode_to_utf8_without_replacement(&[], &mut dest, true);

    valid_result(status)
}

fn valid_result(result: DecoderResult) -> bool {
    match result {
        DecoderResult::InputEmpty => true,
        DecoderResult::OutputFull => unreachable!(),
        DecoderResult::Malformed(_, _) => false,
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn validate_valid() {
        let valid = b"I am valid\xf0\x90\x8c\xbc";
        assert!(validate(valid.iter().cloned()));
    }

    #[test]
    fn validate_invalid() {
        let invalid = b"I am invalid\xc3\x28";
        assert!(!validate(invalid.iter().cloned()));
    }
}
